// Main.cpp : This file contains the 'main' function. Program execution begins and ends there.
//

#include "Yaml.h"
#include <iostream>

using namespace OpenXcom;

static int _v1 = 1;
static int _v2 = 2;
static int _v3 = 3;
static int _v4 = 4;
static int _v5 = 5;
static std::string _v6 = "v6";

void load1(const YAML::YamlNodeReader& reader);
void load2(const YAML::YamlNodeReader& reader);
void save1(YAML::YamlNodeWriter writer);
void save2(YAML::YamlNodeWriter writer);

int main()
{
    YAML::YamlString yaml(R"(
k1: 1
k2: {k3: 3, k4: 4}
k5: [1, 2, 3]
k6: v6 #comment
)"); 
    //writing test
    auto writer = YAML::YamlRootNodeWriter();
    writer.setAsMap();
    writer.write("k1", _v1);
    save1(writer.writeKey("k2"));
    save2(writer.writeVector("k5", std::vector<int>{ 1, 2, 3 }));
    writer.write("k6", _v6);
    std::string yaml2 = writer.emit();
    std::cout << yaml2;

    //reading test
    YAML::YamlRootNodeReader reader(yaml);
    int k1val = reader["k1"].readVal<int>(); //same as YAML::Node().as<int>()
    k1val = reader.read("k1", 1); //same as YAML::Node().as<int>(fallback)
    reader.tryRead("k1", k1val);
    load1(reader["k2"]);
    std::vector<int> vector = reader.read<std::vector<int>>("k5"); //same thing, but with default value
    std::string k6val;
    reader.tryRead("k6", k6val);
    return 0;
}

void load1(const YAML::YamlNodeReader& reader)
{
    const auto& reader2 = reader.useIndex();
    load2(reader2["k4"]);
}

void load2(const YAML::YamlNodeReader& reader)
{
    bool hasVal = reader.hasVal();
    int val = reader.readVal<int>();
}

void save1(YAML::YamlNodeWriter writer)
{
    writer.setAsMap();
    writer.setFlowStyle();
    writer.write("k3", _v3);
    writer.write("k4", _v3);
}

void save2(YAML::YamlNodeWriter writer)
{
    writer.setFlowStyle();
}
